package org.metawidget.forge.view;

import java.util.List;
import java.util.ArrayList;
import org.metawidget.forge.view.PaginationHelper;
import org.metawidget.forge.persistence.PersistenceUtil;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
public abstract class CommonGenericBean<T>
{
   private List<T> list = null;
   private T item = createInstence();
   private long id = 0;
   private PaginationHelper<T> pagination;		
   private transient Class<T> entityClass;   
   
   protected PersistenceUtil persit;
   
   protected  abstract void setPersistenceUnit(PersistenceUtil persit);
   
   public Class<T> getItemType()
   {
	   if (entityClass == null) {
           entityClass = getSuperClassGenricType(getClass());
       }
       return entityClass;
   }   

   public void load()
   {
      item = persit.findById(getItemType(), id);
   }
   
   public String create()
   {
	   persit.create(item);
      return "view?faces-redirect=true&id=" + getEntityId(item);
   }

   public String delete()
   {
	   persit.delete(item);
      return "list?faces-redirect=true";
   }

   public String save()
   {
	   persit.save(item);
      return "view?faces-redirect=true&id=" + getEntityId(item);
   }

   public long getId()
   {
      return id;
   }

   public void setId(long id)
   {
      this.id = id;
      if(id>0){
			load();
	  }	
   }
   
   public T getItem()
   {
      return item;
   }

   public void setItem(T item)
   {
      this.item = item;
   }

   public List<T> getList()
   {
      if(list == null)
      {
         list = getPagination().createPageDataModel();
      }
      return list;
   }

   public void setList(List<T> list)
   {
      this.list = list;
   }
  
   public PaginationHelper<T> getPagination() 
   {
		if (pagination == null) 
		{
			pagination = new PaginationHelper<T>(10) 
			{
				@Override
				public int getItemsCount() {
					return persit.count(getItemType());
				}

				@Override
				public List<T> createPageDataModel() 
				{
					return new ArrayList<T>(persit.findAll(getItemType(),
							 getPageFirstItem(), getPageSize() ));
				}
			};
		}
		return pagination;
	}

   public void setPagination(final PaginationHelper<T> helper)
   {
      pagination = helper;
   }
   protected T createInstence() {
       try {           
           return getItemType().newInstance();
       } catch (InstantiationException e) {
           e.printStackTrace();
       } catch (IllegalAccessException e) {
           e.printStackTrace();
       }
       return null;
   }
 
   private Object getEntityId(T entity) {
       try {
           Method getId = getItemType().getMethod("getId");
           return getId.invoke(entity);
       } catch (SecurityException e) {
           e.printStackTrace();
       } catch (NoSuchMethodException e) {
           e.printStackTrace();
       } catch (IllegalArgumentException e) {
           e.printStackTrace();
       } catch (IllegalAccessException e) {
           e.printStackTrace();
       } catch (InvocationTargetException e) {
           e.printStackTrace();
       }
       return "";
   }
   @SuppressWarnings("unchecked")
   private <T> Class<T> getSuperClassGenricType(final Class clazz) {
       return getSuperClassGenricType(clazz, 0);
   }

   @SuppressWarnings("unchecked")
   private Class getSuperClassGenricType(final Class clazz, final int index) {
       Type genType = clazz.getGenericSuperclass();
       if (!(genType instanceof ParameterizedType)) {
           return Object.class;
       }
       Type[] params = ((ParameterizedType) genType).getActualTypeArguments();
       if (index >= params.length || index < 0) {
           return Object.class;
       }
       if (!(params[index] instanceof Class)) {
           return Object.class;
       }
       return (Class) params[index];
   }
}