package org.metawidget.forge.view;

import java.util.List;

public abstract class PaginationHelper<T>
{
   private final int pageSize;
   private int page;

   public PaginationHelper(int pageSize)
   {
      this.pageSize = pageSize;
   }

   public abstract int getItemsCount();

   public abstract List<T> createPageDataModel();

   public int getPageFirstItem()
   {
      return page * pageSize;
   }

   public int getPageLastItem()
   {
      int i = getPageFirstItem() + pageSize - 1;
      int count = getItemsCount() - 1;
      if (i > count) {
         i = count;
      }
      if (i < 0) {
         i = 0;
      }
      return i;
   }

   public boolean isHasNextPage()
   {
      return (page + 1) * pageSize + 1 <= getItemsCount();
   }

   public boolean isHasPreviousPage()
   {
      return page > 0;
   }

   public int getPageSize()
   {
      return pageSize;
   }

   public int getPage()
   {
      return page;
   }

   public void setPage(int page)
   {
      this.page = page;
   }
}
